#!/usr/bin/env python3
import fileinput
import os
import re
import subprocess

copyright_header = """/\* mockturtle: C\+\+ logic network library
 \* Copyright \(C\) ([\d-]+)  EPFL
 \*
 \* Permission is hereby granted, free of charge, to any person
 \* obtaining a copy of this software and associated documentation
 \* files \(the \"Software\"\), to deal in the Software without
 \* restriction, including without limitation the rights to use,
 \* copy, modify, merge, publish, distribute, sublicense, and/or sell
 \* copies of the Software, and to permit persons to whom the
 \* Software is furnished to do so, subject to the following
 \* conditions:
 \*
 \* The above copyright notice and this permission notice shall be
 \* included in all copies or substantial portions of the Software.
 \*
 \* THE SOFTWARE IS PROVIDED \"AS IS\", WITHOUT WARRANTY OF ANY KIND,
 \* EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 \* OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 \* NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 \* HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 \* WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 \* FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 \* OTHER DEALINGS IN THE SOFTWARE.
 \*/
"""

copyright_header_replace = """/* mockturtle: C++ logic network library
 * Copyright (C) 2018-2022  EPFL
 *
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following
 * conditions:
 *
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 */
"""

file_header_match = """/\*\!
\s+\\\\file\s+([\w.]+)
\s+\\\\brief\s+([^\n\\\\]*)
(\s+\\\\author[^\*]*)\*/"""

file_header_replace = """/*!
  \\file {}
  \\brief {}

{}
*/"""


def find_files( path, ext ):
    items = []
    for root, dirs, files in os.walk( path ):
        for name in files:
            if name.endswith( ext ):
                items.append( os.path.join( root, name ) )
    return items

def read_file( filename ):
    with open( filename, "r" ) as f:
        content = f.read().splitlines()
    return content

def write_file( filename, content ):
    with open( filename, "w" ) as f:
        f.write( '\n'.join( content ) )

def match_replace( content, pattern, replace ):
    found = False
    for match in pattern.finditer( content ):
        found = True
        content = ''.join( [content[:match.start()], replace, content[match.end():]] )
    return found, content

def match_file_header( content, pattern, filename, authors ):
    found = False
    for match in pattern.finditer( content ):
        found = True

        if match[1] != filename:
            print( "[w] filename does not match {} {}".format( match[1], filename ) )

        replace = file_header_replace.format( filename, match[2], "".join( ["  \\author {}\n".format( a ) for a in authors] ).rstrip() )
        content = ''.join( [content[:match.start()], replace, content[match.end():]] )

    return found, content

author_pseudonyms = {
    'Eleonora' : 'Eleonora Testa',
    'b1f6c1c4' : 'Jinzheng Tu',
    'eletesta' : 'Eleonora Testa',
    'gmeuli' : 'Giulia Meuli',
    'lee30sonia' : 'Siang-Yun (Sonia) Lee',
    'mdsudara' : 'Dewmini Sudara',
    'wlneto' : 'Walter Lau Neto',
    'zfchu' : 'Zhufei Chu',
    'Andrea' : 'Andrea Costamagna'
}
def git_authors( file ):
    result = subprocess.check_output(['git', 'shortlog', '-s', '--', file])

    authors = []
    lines = result.decode('utf-8').split('\n')
    for l in lines:
        author_information = l.split( '\t' )
        if len( author_information ) > 1:
            name = author_information[1]
            if ( name in author_pseudonyms ):
                authors.append( author_pseudonyms[name] )
            else:
                authors.append( name )

    # remove duplicate names
    authors = list( dict.fromkeys( authors ) )
    return sorted( authors )

# find all `.hpp` files
hpp_files = find_files( 'include/', '.hpp' )
cpp_files = find_files( 'experiments/', '.cpp' )

# prepare regular expression patterns
copyright_header_pattern = re.compile( copyright_header )
file_header_pattern = re.compile( file_header_match )

for path in hpp_files:
    filename = os.path.basename( path )

    # determine authors of file
    authors = git_authors( path )

    content = read_file( path )
    content = '\n'.join( content )
    new_content = content

    # update copyright header if necessary
    copyright_header_found, new_content = match_replace( new_content, copyright_header_pattern, copyright_header_replace )

    # add copyright header if not found
    if not copyright_header_found:
        new_content = ''.join( [ copyright_header_replace, new_content ] )

    # update file header if necessary
    found, new_content = match_file_header( new_content, file_header_pattern, filename, authors )

    # rewrite the file
    if ( content != new_content ):
        print( "create backup ", path + "~" )
        write_file( path + "~", [ content ] )

        print( "update file ", path )
        write_file( path, [ new_content ] )

for path in cpp_files:
    filename = os.path.basename( path )

    # determine authors of file
    authors = git_authors( path )

    content = read_file( path )
    content = '\n'.join( content )
    new_content = content

    # update copyright header if necessary
    copyright_header_found, new_content = match_replace( new_content, copyright_header_pattern, copyright_header_replace )

    # add copyright header if not found
    if not copyright_header_found:
        new_content = ''.join( [ copyright_header_replace, new_content ] )

    # rewrite the file
    if ( content != new_content ):
        print( "create backup ", path + "~" )
        write_file( path + "~", [ content ] )

        print( "update file ", path )
        write_file( path, [ new_content ] )



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from conans import ConanFile, tools
import os

class SparseppConan(ConanFile):
    name = "parallel_hashmap"
    version = "1.27"
    description = "A header-only, very fast and memory-friendly hash map"
    
    # Indicates License type of the packaged library
    license = "https://github.com/greg7mdp/parallel-hashmap/blob/master/LICENSE"
    
    # Packages the license for the conanfile.py
    exports = ["LICENSE"]
    
    # Custom attributes for Bincrafters recipe conventions
    source_subfolder = "source_subfolder"
    
    def source(self):
        source_url = "https://github.com/greg7mdp/parallel-hashmap"
        tools.get("{0}/archive/{1}.tar.gz".format(source_url, self.version))
        extracted_dir = self.name + "-" + self.version

        #Rename to "source_folder" is a convention to simplify later steps
        os.rename(extracted_dir, self.source_subfolder)


    def package(self):
        include_folder = os.path.join(self.source_subfolder, "parallel_hashmap")
        self.copy(pattern="LICENSE")
        self.copy(pattern="*", dst="include/parallel_hashmap", src=include_folder)

    def package_id(self):
        self.info.header_only()



from __future__ import annotations

import sys

if sys.version_info < (3, 8):  # noqa: UP036
    msg = "pybind11 does not support Python < 3.8. v2.13 was the last release supporting Python 3.7."
    raise ImportError(msg)


from ._version import __version__, version_info
from .commands import get_cmake_dir, get_include, get_pkgconfig_dir

__all__ = (
    "version_info",
    "__version__",
    "get_include",
    "get_cmake_dir",
    "get_pkgconfig_dir",
)



import os
import sys
import platform
import re
import subprocess

from setuptools import setup, Extension, find_namespace_packages
from setuptools.command.build_ext import build_ext


class CMakeExtension(Extension):
    def __init__(self, name, sourcedir='', namespace=''):
        Extension.__init__(self, name, sources=[])
        self.sourcedir = os.path.abspath(sourcedir)
        self.namespace = namespace


class CMakeBuild(build_ext):
    def build_extension(self, ext):
        self.package = ext.namespace
        extdir = os.path.abspath(os.path.dirname(self.get_ext_fullpath(ext.name)))
        # required for auto-detection of auxiliary "native" libs
        if not extdir.endswith(os.path.sep):
            extdir += os.path.sep

        cmake_args = ['-DCMAKE_LIBRARY_OUTPUT_DIRECTORY=' + extdir,
                      '-DPYTHON_EXECUTABLE=' + sys.executable,
                      '-DMOCKTURTLE_EXAMPLES=OFF',
                      '-DAIGVERSE_ENABLE_IPO=ON',
                      '-DAIGVERSE_ENABLE_PCH=ON',
                      '-DAIGVERSE_ENABLE_UNITY_BUILD=ON',
                      ]

        if self.compiler.compiler_type != "msvc":
            # Using Ninja-build since it a) is available as a wheel and b)
            # multithreads automatically. MSVC would require all variables be
            # exported for Ninja to pick it up, which is a little tricky to do.
            # Users can override the generator with CMAKE_GENERATOR in CMake
            # 3.15+.
            cmake_generator = os.environ.get("CMAKE_GENERATOR", "")
            if not cmake_generator:
                cmake_args += ["-GNinja"]

        cfg = 'Debug' if self.debug else 'Release'
        cmake_args += ['-DCMAKE_BUILD_TYPE={}'.format(cfg)]
        build_args = ['--config', cfg]

        if platform.system() == "Windows":
            cmake_args += ['-DCMAKE_LIBRARY_OUTPUT_DIRECTORY_{}={}'.format(cfg.upper(), extdir)]
            cmake_args += ['-T', 'ClangCl']
            if sys.maxsize > 2 ** 32:
                cmake_args += ['-A', 'x64']
            build_args += ['--', '/m']
        else:
            cpus = os.cpu_count()
            if cpus is None:
                cpus = 2
            build_args += ['--', f'-j{cpus}']

        # cross-compile support for macOS - respect ARCHFLAGS if set
        if sys.platform.startswith("darwin"):
            archs = re.findall(r"-arch (\S+)", os.environ.get("ARCHFLAGS", ""))
            if archs:
                arch_argument = "-DCMAKE_OSX_ARCHITECTURES={}".format(";".join(archs))
                print('macOS building with: ', arch_argument, flush=True)
                cmake_args += [arch_argument]

        env = os.environ.copy()
        env['CXXFLAGS'] = '{} -DVERSION_INFO=\\"{}\\"'.format(env.get('CXXFLAGS', ''),
                                                              self.distribution.get_version())
        if not os.path.exists(self.build_temp):
            os.makedirs(self.build_temp)
        else:
            os.remove(os.path.join(self.build_temp, 'CMakeCache.txt'))

        subprocess.check_call(['cmake', ext.sourcedir] + cmake_args, cwd=self.build_temp, env=env)
        subprocess.check_call(['cmake', '--build', '.', '--target', ext.name] + build_args, cwd=self.build_temp)


README_PATH = os.path.join(os.path.abspath(os.path.dirname(__file__)), 'README.md')
with open(README_PATH, encoding="utf8") as readme_file:
    README = readme_file.read()

setup(
    name='aigverse',
    version='0.0.8',
    author='Marcel Walter',
    author_email='marcel.walter@tum.de',
    description='A Python library for working with logic networks, synthesis, and optimization.',
    long_description=README,
    long_description_content_type='text/markdown',
    license='MIT',
    url='https://github.com/marcelwa/aigverse',
    ext_modules=[CMakeExtension('aigverse')],
    cmdclass={"build_ext": CMakeBuild},
    zip_safe=False,
    python_requires=">=3.8",
    classifiers=[
        'Development Status :: 4 - Beta',
        'Programming Language :: Python :: 3',
        'Programming Language :: C++',
        'Environment :: Console',
        'License :: OSI Approved :: MIT License',
        'Operating System :: Microsoft :: Windows',
        'Operating System :: MacOS',
        'Operating System :: POSIX :: Linux',
        'Intended Audience :: Science/Research',
        'Natural Language :: English',
        'Topic :: Scientific/Engineering :: Electronic Design Automation (EDA)',
    ],
    keywords='aigverse logic synthesis AIG optimization',
    project_urls={
        'Source': 'https://github.com/marcelwa/aigverse',
        'Tracker': 'https://github.com/marcelwa/aigverse/issues',
        # 'Documentation': 'https://aigverse.readthedocs.io/en/latest/',
    }
)
